@@ -1,12 +1,18 @@
 package POE::Component::IRC::Plugin::Console;
+BEGIN {
+  $POE::Component::IRC::Plugin::Console::AUTHORITY = 'cpan:HINRIK';
+}
+{
+  $POE::Component::IRC::Plugin::Console::VERSION = '6.83';
+}
 
 use strict;
-use warnings;
+use warnings FATAL => 'all';
 use Carp;
+use IRC::Utils qw(decode_irc);
 use POE qw(Wheel::SocketFactory Wheel::ReadWrite Filter::IRCD Filter::Line Filter::Stackable);
 use POE::Component::IRC::Plugin qw( :ALL );
-
-our $VERSION = '6.22';
+use Scalar::Util qw(looks_like_number);
 
 sub new {
     my $package = shift;
@@ -41,29 +47,54 @@ sub PCI_unregister {
     return 1;
 }
 
-sub _default {
-    my ($self, $irc) = splice @_, 0, 2;
-    my $event = shift;
-    return PCI_EAT_NONE if $event eq 'S_raw';
-    
-    pop @_ if ref $_[-1] eq 'ARRAY';
-    my @args = map { $$_ } @_;
-    my @output = ( "$event: " );
+sub _dump {
+    my ($arg) = @_;
 
-    for my $arg ( @args ) {
-        if ( ref($arg) eq 'ARRAY' ) {
-            push( @output, '[' . join(', ', @$arg ) . ']' );
+    if (ref $arg eq 'ARRAY') {
+        my @elems;
+        for my $elem (@$arg) {
+            push @elems, _dump($elem);
         }
-        else {
-            push ( @output, "'$arg'" );
+        return '['. join(', ', @elems) .']';
+    }
+    elsif (ref $arg eq 'HASH') {
+        my @pairs;
+        for my $key (keys %$arg) {
+            push @pairs, [$key, _dump($arg->{$key})];
         }
+        return '{'. join(', ', map { "$_->[0] => $_->[1]" } @pairs) .'}';
+    }
+    elsif (ref $arg) {
+        require overload;
+        return overload::StrVal($arg);
+    }
+    elsif (defined $arg) {
+        return $arg if looks_like_number($arg);
+        return "'".decode_irc($arg)."'";
+    }
+    else {
+        return 'undef';
+    }
+}
+
+sub _default {
+    my ($self, $irc, $event) = splice @_, 0, 3;
+    return PCI_EAT_NONE if $event eq 'S_raw';
+
+    pop @_;
+    my @args = map { $$_ } @_;
+    my @output;
+
+    for my $i (0..$#args) {
+        push @output, "ARG$i: " . _dump($args[$i]);
     }
 
     for my $wheel_id ( keys %{ $self->{wheels} } ) {
         next if ( $self->{exit}->{ $wheel_id } or ( not defined ( $self->{wheels}->{ $wheel_id } ) ) );
-        $self->{wheels}->{ $wheel_id }->put( join(' ', @output ) );
+        next if !$self->{authed}{ $wheel_id };
+        $self->{wheels}->{ $wheel_id }->put("$event: ".join(', ', @output));
     }
-    
+
     return PCI_EAT_NONE;
 }
 
@@ -84,14 +115,14 @@ sub _start {
         FailureEvent => '_listener_failed',
         Reuse        => 'yes',
     );
-    
+
     if ($self->{listener}) {
         $self->{irc}->send_event( 'irc_console_service' => $self->{listener}->getsockname() );
     }
     else {
         $self->{irc}->plugin_del( $self );
     }
-    
+
     return;
 }
 
@@ -149,7 +180,7 @@ sub _client_input {
         $self->{irc}->send_event( 'irc_console_authed' => $wheel_id );
         return;
     }
-    
+
     $self->{wheels}->{ $wheel_id }->put('NOTICE * Password required * enter PASS <password> *');
     return;
 }
@@ -186,7 +217,8 @@ sub getsockname {
 }
 
 1;
-__END__
+
+=encoding utf8
 
 =head1 NAME
 
@@ -205,7 +237,7 @@ lightweight debugging and control console for your bot
 
  my @channels = ( '#Blah', '#Foo', '#Bar' );
 
- my $irc = POE::Component::IRC->spawn( 
+ my $irc = POE::Component::IRC->spawn(
      nick => $nickname,
      server => $ircserver,
      port => $port,
@@ -222,7 +254,7 @@ lightweight debugging and control console for your bot
  $poe_kernel->run();
 
  sub _start {
-     $irc->plugin_add( 'Console' => POE::Component::IRC::Plugin::Console->new( 
+     $irc->plugin_add( 'Console' => POE::Component::IRC::Plugin::Console->new(
          bindport => $bindport,
          password => 'opensesame'
      );
@@ -242,7 +274,7 @@ lightweight debugging and control console for your bot
  }
 
  sub irc_console_connect {
-     my ($peeradr, $peerport, $wheel_id) = @_[ARG0, .. ARG2];
+     my ($peeradr, $peerport, $wheel_id) = @_[ARG0 .. ARG2];
      return;
  }
 
@@ -289,7 +321,7 @@ L<POE::Component::IRC|POE::Component::IRC>'s C<plugin_add> method.
 Gives access to the underlying listener's C<getsockname> method. See
 L<POE::Wheel::SocketFactory|POE::Wheel::SocketFactory> for details.
 
-=head1 OUTPUT
+=head1 OUTPUT EVENTS
 
 The plugin generates the following additional
 L<POE::Component::IRC|POE::Component::IRC> events:
