@@ -2,18 +2,20 @@ package POE::Component::IRC::Plugin::DCC;
 BEGIN {
   $POE::Component::IRC::Plugin::DCC::AUTHORITY = 'cpan:HINRIK';
 }
-BEGIN {
-  $POE::Component::IRC::Plugin::DCC::VERSION = '6.35';
+{
+  $POE::Component::IRC::Plugin::DCC::VERSION = '6.83';
 }
 
 use strict;
-use warnings;
+use warnings FATAL => 'all';
 use Carp;
 use File::Basename qw(fileparse);
+use File::Glob ':glob';
+use File::Spec::Functions 'rel2abs';
 use POE qw(Driver::SysRW Filter::Line Filter::Stream
            Wheel::ReadWrite Wheel::SocketFactory);
 use POE::Component::IRC::Plugin qw(:ALL);
-use Socket;
+use Socket qw(INADDR_ANY unpack_sockaddr_in inet_aton inet_ntoa);
 
 use constant {
     OUT_BLOCKSIZE  => 1024,   # Send DCC data in 1k chunks
@@ -30,9 +32,9 @@ sub new {
 
 sub PCI_register {
     my ($self, $irc) = @_;
-    
+
     $self->{irc} = $irc;
-    
+
     POE::Session->create(
         object_states => [
             $self => [qw(
@@ -46,13 +48,14 @@ sub PCI_register {
                 _U_dcc_chat
                 _U_dcc_close
                 _U_dcc_resume
+                _cancel_timeout
             )],
         ],
     );
 
     $irc->plugin_register($self, 'SERVER', qw(disconnected dcc_request));
     $irc->plugin_register($self, 'USER', qw(dcc dcc_accept dcc_chat dcc_close dcc_resume));
-    
+
     return 1;
 }
 
@@ -88,7 +91,7 @@ sub nataddr {
 # returns information about a connection
 sub dcc_info {
     my ($self, $id) = @_;
-    
+
     if (!$self->{dcc}->{$id}) {
         warn "dcc_info: Unknown wheel ID: $id\n";
         return;
@@ -114,7 +117,7 @@ sub _quote_file {
 
 sub S_disconnected {
     my ($self) = $_;
-    # clean up old cookies for any ignored RESUME requests 
+    # clean up old cookies for any ignored RESUME requests
     delete $self->{resuming};
     return PCI_EAT_NONE;
 }
@@ -134,6 +137,7 @@ sub S_dcc_request {
             next if $cookie->{nick} ne $nick;
             next if $cookie->{port} ne $port;
             $file = _quote_file($file);
+            $cookie->{done} = $size;
             $irc->yield(ctcp => $nick => "DCC ACCEPT $file $port $size");
             last;
         }
@@ -147,8 +151,9 @@ sub S_dcc_request {
 # namely create wheels and set alarms/delays
 sub _default {
     my ($self, $irc, $event) = splice @_, 0, 3;
-    return PCI_EAT_NONE if $event !~ /^U_dcc(_accept|_chat|_close|_resume)?$/;
+    return PCI_EAT_NONE if $event !~ /^U_dcc(?:_accept|_chat|_close|_resume)?$/;
     $event =~ s/^U_/_U_/;
+    pop @_;
     my @args = map { $$_ } @_;
     $poe_kernel->call($self->{session_id}, $event, @args);
     return PCI_EAT_NONE;
@@ -158,7 +163,7 @@ sub _default {
 sub _U_dcc {
     my ($kernel, $self, $nick, $type, $file, $blocksize, $timeout)
         = @_[KERNEL, OBJECT, ARG0..$#_];
-    
+
     if (!defined $type) {
         warn "The 'dcc' command requires at least two arguments\n";
         return;
@@ -176,6 +181,7 @@ sub _U_dcc {
             warn "The 'dcc' command requires three arguments for a SEND\n";
             return;
         }
+        $file = rel2abs(bsd_glob($file));
         $size = (stat $file)[7];
         if (!defined $size) {
             $irc->send_event(
@@ -190,7 +196,7 @@ sub _U_dcc {
             return;
         }
     }
-    
+
     $bindaddr = $irc->localaddr();
 
     if ($self->{dccports}) {
@@ -208,10 +214,10 @@ sub _U_dcc {
         FailureEvent => '_dcc_failed',
         Reuse        => 'yes',
     );
-    
+
     ($port, $addr) = unpack_sockaddr_in($factory->getsockname());
     $addr = inet_aton($self->{nataddr}) if $self->{nataddr};
-  
+
     if (!defined $addr) {
         warn "dcc: Can't determine our IP address! ($!)\n";
         return;
@@ -224,6 +230,10 @@ sub _U_dcc {
     # Tell the other end that we're waiting for them to connect.
     $irc->yield(ctcp => $nick => "DCC $type $basename $addr $port" . ($size ? " $size" : ''));
 
+    my $alarm_id = $kernel->delay_set(
+        '_dcc_timeout', ($timeout || LISTEN_TIMEOUT), $factory->ID,
+    );
+
     # Store the state for this connection.
     $self->{dcc}->{ $factory->ID } = {
         open      => 0,
@@ -237,14 +247,9 @@ sub _U_dcc {
         blocksize => ($blocksize || OUT_BLOCKSIZE),
         listener  => 1,
         factory   => $factory,
+        alarm_id  => $alarm_id,
     };
-    
-    $kernel->alarm(
-        '_dcc_timeout',
-        time() + ($timeout || LISTEN_TIMEOUT),
-        $factory->ID,
-    );
-    
+
     return;
 }
 
@@ -269,7 +274,7 @@ sub _U_dcc_accept {
         SuccessEvent  => '_dcc_up',
         FailureEvent  => '_dcc_failed',
     );
-  
+
     $self->{dcc}->{$factory->ID} = $cookie;
     $self->{dcc}->{$factory->ID}->{factory} = $factory;
 
@@ -279,7 +284,7 @@ sub _U_dcc_accept {
 # Send data over a DCC CHAT connection.
 sub _U_dcc_chat {
     my ($self, $id, @data) = @_[OBJECT, ARG0..$#_];
-    
+
     if (!defined $id || !@data) {
         warn "The 'dcc_chat' command requires at least two arguments\n";
         return;
@@ -289,12 +294,12 @@ sub _U_dcc_chat {
         warn "dcc_chat: Unknown wheel ID: $id\n";
         return;
     }
-    
+
     if (!exists $self->{dcc}->{$id}->{wheel}) {
         warn "dcc_chat: No DCC wheel for id $id!\n";
         return;
     }
-  
+
     if ($self->{dcc}->{$id}->{type} ne 'CHAT') {
         warn "dcc_chat: id $id isn't associated with a DCC CHAT connection!\n";
         return;
@@ -308,7 +313,7 @@ sub _U_dcc_chat {
 sub _U_dcc_close {
     my ($kernel, $self, $id) = @_[KERNEL, OBJECT, ARG0];
     my $irc = $self->{irc};
-    
+
     if (!defined $id) {
         warn "The 'dcc_close' command requires an id argument\n";
         return;
@@ -318,7 +323,7 @@ sub _U_dcc_close {
         warn "dcc_close: Unknown wheel ID: $id\n";
         return;
     }
-    
+
     if (!exists $self->{dcc}->{$id}->{wheel}) {
         warn "dcc_close: No DCC wheel for id $id!\n";
         return;
@@ -343,12 +348,7 @@ sub _U_dcc_close {
         push ( @{ $self->{dccports} }, $self->{dcc}->{$id}->{port} );
     }
 
-    if (exists $self->{dcc}->{$id}->{wheel}) {
-        delete $self->{wheelmap}->{$self->{dcc}->{$id}->{wheel}->ID};
-        delete $self->{dcc}->{$id}->{wheel};
-    }
-
-    delete $self->{dcc}->{$id};
+    $self->_remove_dcc($id);
     return;
 }
 
@@ -359,33 +359,30 @@ sub _U_dcc_resume {
 
     my $sender_file = _quote_file($cookie->{file});
     $cookie->{file} = $myfile if defined $myfile;
-    my $size = -s $cookie->{file};
-    my $fraction = $size % IN_BLOCKSIZE;
-    $size -= $fraction;
+    $cookie->{done} = -s $cookie->{file};
+    $cookie->{resuming} = 1;
 
-    # we need to truncate the whole thing, adjust the size we are
-    # requesting to the size we will truncate the file to
     if (open(my $handle, '>>', $cookie->{file})) {
-        if (!truncate($handle, $size)) {
-            warn "dcc_resume: Can't truncate '$cookie->{file}' to size $size\n";
-            return;
-        }
-        
-        $irc->yield(ctcp => $cookie->{nick} => "DCC RESUME $sender_file $cookie->{port} $size");
-        
-        # save the cookie for later
+        $irc->yield(ctcp => $cookie->{nick} => "DCC RESUME $sender_file $cookie->{port} $cookie->{done}");
         $self->{resuming}->{"$cookie->{port}+$cookie->{nick}"} = $cookie;
     }
-    
+    else {
+        warn "dcc_resume: Can't append to file '$cookie->{file}'\n";
+        return;
+    }
+
     return;
 }
 
 # Accept incoming data on a DCC socket.
 sub _dcc_read {
-    my ($self, $data, $id) = @_[OBJECT, ARG0, ARG1];
+    my ($kernel, $self, $data, $id) = @_[KERNEL, OBJECT, ARG0, ARG1];
     my $irc = $self->{irc};
 
     $id = $self->{wheelmap}->{$id};
+    if ($self->{dcc}{$id}{alarm_id}) {
+        $kernel->call($self->{session_id}, '_cancel_timeout', $id);
+    }
 
     if ($self->{dcc}->{$id}->{type} eq 'GET') {
         # Acknowledge the received data.
@@ -407,7 +404,7 @@ sub _dcc_read {
     elsif ($self->{dcc}->{$id}->{type} eq 'SEND') {
         # Record the client's download progress.
         $self->{dcc}->{$id}->{done} = unpack 'N', substr( $data, -4 );
-        
+
         $irc->send_event(
             'irc_dcc_send',
             $id,
@@ -430,8 +427,8 @@ sub _dcc_read {
                     nick type port file size done peeraddr
                 )},
             );
-            delete $self->{wheelmap}->{ $self->{dcc}->{$id}->{wheel}->ID };
-            delete $self->{dcc}->{$id};
+
+            $self->_remove_dcc($id);
             return;
         }
 
@@ -449,7 +446,7 @@ sub _dcc_read {
             $self->{dcc}->{$id}->{peeraddr},
         );
     }
-    
+
     return;
 }
 
@@ -467,7 +464,7 @@ sub _dcc_failed {
             return;
         }
     }
-  
+
     # Reclaim our port if necessary.
     if ( $self->{dcc}->{$id}->{listener} && $self->{dccports}) {
         push ( @{ $self->{dccports} }, $self->{dcc}->{$id}->{port} );
@@ -475,7 +472,7 @@ sub _dcc_failed {
 
     DCC: {
         last DCC if $errnum != 0;
-    
+
         # Did the peer of a DCC GET connection close the socket after the file
         # transfer finished? If so, it's not really an error.
         if ($self->{dcc}->{$id}->{type} eq 'GET') {
@@ -493,15 +490,12 @@ sub _dcc_failed {
                 )},
             );
 
-            if ( $self->{dcc}->{$id}->{wheel} ) {
-                delete $self->{wheelmap}->{ $self->{dcc}->{$id}->{wheel}->ID };
-            }
-            delete $self->{dcc}->{$id};
+            $self->_remove_dcc($id);
         }
-        
+
         return;
     }
-  
+
     # something went wrong
     if ($errnum == 0 && $self->{dcc}->{$id}->{type} eq 'GET') {
         $errstr = 'Aborted by sender';
@@ -512,7 +506,7 @@ sub _dcc_failed {
             : $errstr = "$operation error $errnum"
         ;
     }
-  
+
     $irc->send_event(
         'irc_dcc_error',
         $id,
@@ -522,11 +516,7 @@ sub _dcc_failed {
         )},
     );
 
-    if (exists $self->{dcc}->{$id}->{wheel}) {
-        delete $self->{wheelmap}->{$self->{dcc}->{$id}->{wheel}->ID};
-    }
-    delete $self->{dcc}->{$id};
-
+    $self->_remove_dcc($id);
     return;
 }
 
@@ -553,7 +543,7 @@ sub _dcc_up {
     my ($kernel, $self, $sock, $peeraddr, $id) =
         @_[KERNEL, OBJECT, ARG0, ARG1, ARG3];
     my $irc = $self->{irc};
-    
+
     # Delete the listening socket and monitor the accepted socket
     # for incoming data
     delete $self->{dcc}->{$id}->{factory};
@@ -573,19 +563,19 @@ sub _dcc_up {
         InputEvent => '_dcc_read',
         ErrorEvent => '_dcc_failed',
     );
-    
+
     $self->{wheelmap}->{ $self->{dcc}->{$id}->{wheel}->ID } = $id;
-    
+
     my $handle;
     if ($self->{dcc}->{$id}->{type} eq 'GET') {
         # check if we're resuming
-        my $mode = -s $self->{dcc}->{$id}->{file} ? '>>' : '>';
-        
+        my $mode = $self->{dcc}->{$id}->{resuming} ? '>>' : '>';
+
         if ( !open $handle, $mode, $self->{dcc}->{$id}->{file} ) {
             $kernel->yield(_dcc_failed => 'open file', $! + 0, $!, $id);
             return;
         }
-        
+
         binmode $handle;
         $self->{dcc}->{$id}->{fh} = $handle;
     }
@@ -596,6 +586,7 @@ sub _dcc_up {
         }
 
         binmode $handle;
+        seek $handle, $self->{dcc}{$id}{done}, 0;
         # Send the first packet to get the ball rolling.
         read $handle, my $buffer, $self->{dcc}->{$id}->{blocksize};
         $self->{dcc}->{$id}->{wheel}->put($buffer);
@@ -610,12 +601,39 @@ sub _dcc_up {
             nick type port file size peeraddr
         )},
     );
-    
+
+    return;
+}
+
+sub _cancel_timeout {
+    my ($kernel, $self, $id) = @_[KERNEL, OBJECT, ARG0];
+    my $alarm_id = delete $self->{dcc}{$id}{alarm_id};
+    $kernel->alarm_remove($alarm_id);
+    return;
+}
+
+sub _remove_dcc {
+    my ($self, $id) = @_;
+
+    if (exists $self->{dcc}{$id}{alarm_id}) {
+        $poe_kernel->call($self->{session_id}, '_cancel_timeout', $id);
+    }
+
+    if (exists $self->{dcc}{$id}{wheel}) {
+        delete $self->{wheelmap}{ $self->{dcc}{$id}{wheel}->ID };
+        if ($^O =~ /cygwin|MSWin/) {
+          $self->{dcc}{$id}{wheel}->$_ for qw(shutdown_input shutdown_output);
+        }
+    }
+
+    # flush the filehandle
+    close $self->{dcc}{$id}{fh} if $self->{dcc}{$id}{type} eq 'GET';
+
+    delete $self->{dcc}{$id};
     return;
 }
 
 1;
-__END__
 
 =encoding utf8
 
@@ -691,43 +709,43 @@ value in seconds (default: 300).
 Incidentally, you can send other weird nonstandard kinds of DCCs too;
 just put something besides 'SEND' or 'CHAT' (say, 'FOO') in the type
 field, and you'll get back C<irc_dcc_foo> events (with the same arguments as
-L<C<irc_dcc_chat>|/"irc_dcc_chat">) when data arrives on its DCC connection.
+L<C<irc_dcc_chat>|/irc_dcc_chat>) when data arrives on its DCC connection.
 
 If you are behind a firewall or Network Address Translation, you may want to
 consult L<POE::Component::IRC|POE::Component::IRC>'s
-L<C<connect>|POE::Component::IRC/"spawn"> for some parameters that are
+L<C<connect>|POE::Component::IRC/spawn> for some parameters that are
 useful with this command.
 
 =head2 C<dcc_accept>
 
 Accepts an incoming DCC connection from another host. First argument:
-the magic cookie from an L<C<irc_dcc_request>|/"irc_dcc_request"> event.
+the magic cookie from an L<C<irc_dcc_request>|/irc_dcc_request> event.
 In the case of a DCC GET, the second argument can optionally specify a
 new name for the destination file of the DCC transfer, instead of using
-the sender's name for it. (See the L<C<irc_dcc_request>|/"irc_dcc_request">
+the sender's name for it. (See the L<C<irc_dcc_request>|/irc_dcc_request>
 section below for more details.)
 
 =head2 C<dcc_resume>
 
 Resumes a DCC SEND file transfer. First argument: the magic cookie from an
-L<C<irc_dcc_request>|/"irc_dcc_request"> event. An optional second argument
+L<C<irc_dcc_request>|/irc_dcc_request> event. An optional second argument
 provides the name of the file to which you want to write.
 
 =head2 C<dcc_chat>
 
 Sends lines of data to the person on the other side of a DCC CHAT connection.
 The first argument should be the wheel id of the connection which you got
-from an L<C<irc_dcc_start>|/"irc_dcc_start"> event, followed by all the data
+from an L<C<irc_dcc_start>|/irc_dcc_start> event, followed by all the data
 you wish to send (it'll be separated with newlines for you).
 
 =head2 C<dcc_close>
 
 Terminates a DCC SEND or GET connection prematurely, and causes DCC CHAT
 connections to close gracefully. Takes one argument: the wheel id of the
-connection which you got from an L<C<irc_dcc_start>|/"irc_dcc_start">
+connection which you got from an L<C<irc_dcc_start>|/irc_dcc_start>
 (or similar) event.
 
-=head1 OUTPUT
+=head1 OUTPUT EVENTS
 
 =head2 C<irc_dcc_request>
 
@@ -737,13 +755,13 @@ to keep all the DCC documentation in one place. In case you were wondering.
 
 You receive this event when another IRC client sends you a DCC
 (e.g. SEND or CHAT) request out of the blue. You can examine the request
-and decide whether or not to accept it (with L<C<dcc_accept>|/"dcc_accept">)
+and decide whether or not to accept it (with L<C<dcc_accept>|/dcc_accept>)
 here. In the case of DCC SENDs, you can also request to resume the file with
-L<C<dcc_resume>|/"dcc_resume">.
+L<C<dcc_resume>|/dcc_resume>.
 
 B<Note:> DCC doesn't provide a way to explicitly reject requests, so if you
 don't intend to accept one, just ignore it or send a
-L<NOTICE|POE::Component::IRC/"notice"> or L<PRIVMSG|POE::Component::IRC/"privmsg">
+L<NOTICE|POE::Component::IRC/notice> or L<PRIVMSG|POE::Component::IRC/privmsg>
 to the peer explaining why you're not going to accept.
 
 =over 4
@@ -856,7 +874,7 @@ connection.
 =head2 C<irc_dcc_done>
 
 You receive this event when a DCC connection terminates normally.
-Abnormal terminations are reported by L<C<irc_dcc_error>|/"irc_dcc_error">.
+Abnormal terminations are reported by L<C<irc_dcc_error>|/irc_dcc_error>.
 
 =over 4
 
