@@ -2,17 +2,17 @@ package POE::Component::IRC::Plugin::PlugMan;
 BEGIN {
   $POE::Component::IRC::Plugin::PlugMan::AUTHORITY = 'cpan:HINRIK';
 }
-BEGIN {
-  $POE::Component::IRC::Plugin::PlugMan::VERSION = '6.35';
+{
+  $POE::Component::IRC::Plugin::PlugMan::VERSION = '6.83';
 }
 
 use strict;
-use warnings;
+use warnings FATAL => 'all';
 use Carp;
+use IRC::Utils qw( matches_mask parse_user );
 use POE::Component::IRC::Plugin qw( :ALL );
-use POE::Component::IRC::Common qw( matches_mask parse_user );
 
-BEGIN { 
+BEGIN {
     # Turn on the debugger's symbol source tracing
     $^P |= 0x10;
 
@@ -39,7 +39,7 @@ sub PCI_register {
 
     $self->{irc} = $irc;
     $irc->plugin_register( $self, 'SERVER', qw(public msg) );
-    
+
     $self->{commands} = {
         PLUGIN_ADD => sub {
             my ($self, $method, $recipient, @cmd) = @_;
@@ -95,11 +95,11 @@ sub S_public {
 
     my (@cmd) = split(/ +/, $command);
     my $cmd = uc (shift @cmd);
-    
+
     if (defined $self->{commands}->{$cmd}) {
         $self->{commands}->{$cmd}->($self, 'privmsg', $channel, @cmd);
     }
-    
+
     return PCI_EAT_NONE;
 }
 
@@ -111,9 +111,9 @@ sub S_msg {
     my $command = ${ $_[2] };
     my (@cmd)   = split(/ +/,$command);
     my $cmd     = uc (shift @cmd);
-    
+
     return PCI_EAT_NONE if !$self->_authed($who, $channel);
-    
+
     if (defined $self->{commands}->{$cmd}) {
         $self->{commands}->{$cmd}->($self, 'notice', $nick, @cmd);
     }
@@ -128,10 +128,10 @@ sub S_msg {
 sub load {
     my ($self, $desc, $plugin) = splice @_, 0, 3;
     return if !$desc || !$plugin;
-    
+
     my $object;
     my $module = ref $plugin || $plugin;
-    if (! ref $plugin){        
+    if (! ref $plugin){
         $module .= '.pm' if $module !~ /\.pm$/;
         $module =~ s/::/\//g;
 
@@ -148,7 +148,7 @@ sub load {
         $object = $plugin;
         $plugin = ref $object;
     }
-    
+
     my $args = [ @_ ];
     $self->{plugins}->{ $desc }->{module} = $module;
     $self->{plugins}->{ $desc }->{plugin} = $plugin;
@@ -162,7 +162,7 @@ sub load {
         # Cleanup
         delete $self->{plugins}->{ $desc };
     }
-    
+
     return $return;
 }
 
@@ -221,7 +221,6 @@ sub _authed {
 }
 
 1;
-__END__
 
 =encoding utf8
 
@@ -240,8 +239,8 @@ management services.
  my $botowner = 'somebody!*@somehost.com';
  my $irc = POE::Component::IRC::State->spawn();
 
- POE::Session->create( 
-     package_states => [ 
+ POE::Session->create(
+     package_states => [
          main => [ qw(_start irc_plugin_add) ],
      ],
  );
@@ -254,7 +253,7 @@ management services.
 
  sub irc_plugin_add {
      my ($desc, $plugin) = @_[ARG0, ARG1];
-     
+
      if ($desc eq 'PlugMan') {
          $plugin->load( 'Connector', 'POE::Component::IRC::Plugin::Connector' );
      }
@@ -282,7 +281,7 @@ may issue commands via the IRC interface. Overrides B<'botowner'>. It will be
 called with three arguments: the IRC component object, the nick!user@host and
 the channel name as arguments. It should return a true value if the user is
 authorized, a false one otherwise.
- 
+
 B<'debug'>, set to a true value to see when stuff goes wrong;
 
 Not setting B<'botowner'> or B<'auth_sub'> effectively disables the IRC
@@ -335,22 +334,22 @@ Returns a list of descriptors of managed plugins.
 =head1 INPUT
 
 An IRC interface is enabled by specifying a "botowner" mask to
-L<C<new>|/"new">. Commands may be either invoked via a PRIVMSG directly to
+L<C<new>|/new>. Commands may be either invoked via a PRIVMSG directly to
 your bot or in a channel by prefixing the command with the nickname of your
 bot. One caveat, the parsing of the irc command is very rudimentary (it
 merely splits the line on spaces).
 
 =head2 C<plugin_add>
 
-Takes the same arguments as L<C<load>|/"load">.
+Takes the same arguments as L<C<load>|/load>.
 
 =head2 C<plugin_del>
 
-Takes the same arguments as L<C<unload>|/"unload">.
+Takes the same arguments as L<C<unload>|/unload>.
 
 =head2 C<plugin_reload>
 
-Takes the same arguments as L<C<reload>|/"reload">.
+Takes the same arguments as L<C<reload>|/reload>.
 
 =head2 C<plugin_loaded>
 
